SystemOrganization addCategory: #'TiledMorph-Demo-SlidePuzzle'!

TiledMorph subclass: #SlidePuzzleGame
	instanceVariableNames: 'emptyTile sem'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TiledMorph-Demo-SlidePuzzle'!

!SlidePuzzleGame class methodsFor: 'as yet unclassified' stamp: 'SebastianNozzi 10/6/2012 20:45'!
new: columns by: rows  forFileNamed: fileName
	^ (self new) initializeWithFileName: fileName columns: columns andRows: rows.! !

!SlidePuzzleGame methodsFor: 'querying' stamp: 'SebastianNozzi 11/1/2012 13:01'!
allNonEmptyTiles
	^ positionedTiles select: [:tile | tile ~= emptyTile].! !

!SlidePuzzleGame methodsFor: 'querying' stamp: 'SebastianNozzi 10/4/2012 17:39'!
canEmptyPositionBeSlidIntoBy: aTile
	^ aTile isAdjacentTo: emptyTile.! !

!SlidePuzzleGame methodsFor: 'event handling' stamp: 'SebastianNozzi 11/1/2012 18:03'!
handleKeystroke: keyEvent 
	| tileToSlide  |
	tileToSlide := keyEvent keyCharacter caseOf: { 
		[Character arrowUp] -> [emptyTile tileBelow ]. 
		[Character arrowDown] -> [emptyTile tileAbove ]. 
		[Character arrowLeft] -> [emptyTile tileToTheRight ]. 
		[Character arrowRight] -> [emptyTile tileToTheLeft ]. 
	}.
	tileToSlide ifNotNil: [tileToSlide tryToSlideIntoEmptyPositionAnimating: true].! !

!SlidePuzzleGame methodsFor: 'initialization' stamp: 'SebastianNozzi 10/7/2012 00:51'!
initialize
	"Do nothing. Use specialized initialization instead"! !

!SlidePuzzleGame methodsFor: 'initialization' stamp: 'SebastianNozzi 10/7/2012 01:29'!
initializeEmptyTile
	self makeTileEmptyAt: 1@1! !

!SlidePuzzleGame methodsFor: 'initialization' stamp: 'SebastianNozzi 11/2/2012 19:26'!
initializeWithColumns: amountOfColumns andRows: amountOfRows
	super initializeWithColumns: amountOfColumns  andRows: amountOfRows.
	sem := Semaphore forMutualExclusion.! !

!SlidePuzzleGame methodsFor: 'initialization' stamp: 'SebastianNozzi 10/7/2012 11:54'!
initializeWithFileName: fileName columns: columns andRows: rows
	self initializeWithColumns: columns andRows: rows.
	self loadTiles: columns by: rows fromFileNamed: fileName.
	self backgroundColor: (Color black).
	self borderColor: (Color darkGray).! !

!SlidePuzzleGame methodsFor: 'querying' stamp: 'SebastianNozzi 10/7/2012 11:51'!
isSolved
	^ self allNonEmptyTiles allSatisfy: [:tile | tile isInStartingPosition ] .! !

!SlidePuzzleGame methodsFor: 'initialization' stamp: 'SebastianNozzi 10/6/2012 20:53'!
loadTiles: columns by: rows fromFileNamed: fileName
	| slicer |
	slicer := ImageSlicer new: columns by: rows forFileNamed: fileName.
	self columnWidth: slicer tileWidth.
	self rowHeight: slicer tileHeight.
	self
		allTilePositionsDo: [ :position | 
			self 
				at: position
				 putTile: (SlidePuzzleTile 
					newWithForm: (slicer imageSliceAt: position)) ].
! !

!SlidePuzzleGame methodsFor: 'actions' stamp: 'SebastianNozzi 10/7/2012 01:29'!
makeTileEmptyAt: tilePosition
	emptyTile := Tile new.
	self at: tilePosition putTile: emptyTile.
! !

!SlidePuzzleGame methodsFor: 'event handling' stamp: 'SebastianNozzi 10/5/2012 19:37'!
mouseDown: anEvent 
	self comeToFront.
	super mouseDown: anEvent.! !

!SlidePuzzleGame methodsFor: 'actions' stamp: 'SebastianNozzi 10/7/2012 19:01'!
reset
	self allNonEmptyTiles do: [:tile | 
		self swapTile: tile  with: (self tileAt: tile startingPosition)]! !

!SlidePuzzleGame methodsFor: 'actions' stamp: 'SebastianNozzi 11/2/2012 19:42'!
shuffle
	self shuffleTimes: 900.! !

!SlidePuzzleGame methodsFor: 'actions' stamp: 'SebastianNozzi 11/2/2012 19:42'!
shuffleOnce
	(emptyTile adjacentTiles atRandom) tryToSlideIntoEmptyPositionAnimating: false ! !

!SlidePuzzleGame methodsFor: 'actions' stamp: 'SebastianNozzi 10/4/2012 17:59'!
shuffleTimes: repetitions
	repetitions timesRepeat: [ self shuffleOnce ] .! !

!SlidePuzzleGame methodsFor: 'actions' stamp: 'SebastianNozzi 11/1/2012 18:02'!
slideTile: aTile animating: animate
	self swapTile: aTile with: emptyTile animating: animate! !

ImageTile subclass: #SlidePuzzleTile
	instanceVariableNames: 'startingPosition'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TiledMorph-Demo-SlidePuzzle'!

!SlidePuzzleTile methodsFor: 'as yet unclassified' stamp: 'SebastianNozzi 10/7/2012 11:50'!
attachTo: aParent
	super attachTo: aParent.
	startingPosition := startingPosition ifNil: [aParent positionOfTile: self].! !

!SlidePuzzleTile methodsFor: 'as yet unclassified' stamp: 'SebastianNozzi 10/7/2012 11:50'!
initialize
	super initialize.
	startingPosition := nil.! !

!SlidePuzzleTile methodsFor: 'as yet unclassified' stamp: 'SebastianNozzi 10/7/2012 11:51'!
isInStartingPosition
	^ startingPosition = (self position)! !

!SlidePuzzleTile methodsFor: 'queying' stamp: 'SebastianNozzi 10/4/2012 17:04'!
maySlideIntoEmptyPosition
	^ parent canEmptyPositionBeSlidIntoBy: self.! !

!SlidePuzzleTile methodsFor: 'event-handling' stamp: 'SebastianNozzi 11/1/2012 18:03'!
mouseDown: anEvent
	self tryToSlideIntoEmptyPositionAnimating: true! !

!SlidePuzzleTile methodsFor: 'as yet unclassified' stamp: 'SebastianNozzi 11/2/2012 16:53'!
printPositionOn: stream
	stream nextPutAll: ' startPos: ', (self startingPosition asString).! !

!SlidePuzzleTile methodsFor: 'accessing' stamp: 'SebastianNozzi 10/7/2012 19:01'!
startingPosition
	^ startingPosition! !

!SlidePuzzleTile methodsFor: 'as yet unclassified' stamp: 'SebastianNozzi 11/1/2012 18:02'!
tryToSlideIntoEmptyPositionAnimating: animate 
	self maySlideIntoEmptyPosition
		ifTrue: [ parent slideTile: self animating: animate ]! !
